@@ -1,5 +1,5 @@
 /* chroot -- run command or shell with special root directory
-   Copyright (C) 95, 96, 1997, 1999 Free Software Foundation, Inc.
+   Copyright (C) 95, 96, 1997, 1999, 2000, 2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "system.h"
 #include "long-options.h"
 #include "error.h"
+#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "chroot"
@@ -42,18 +43,20 @@ usage (int status)
   else
     {
       printf (_("\
-Usage: %s [OPTION] NEWROOT [COMMAND...]\n\
+Usage: %s NEWROOT [COMMAND...]\n\
   or:  %s OPTION\n\
 "), program_name, program_name);
-      printf (_("\
+      fputs (_("\
 Run COMMAND with root directory set to NEWROOT.\n\
 \n\
-      --help       display this help and exit\n\
-      --version    output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 If no command is given, run ``${SHELL} -i'' (default: /bin/sh).\n\
-"));
-      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -66,6 +69,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
 		      AUTHORS, usage);
   if (argc == 1)
