@@ -1,5 +1,5 @@
 /* expr -- evaluate expressions.
-   Copyright (C) 86, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 86, 1991-1997, 1999-2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,13 +31,20 @@
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
-#include <regex.h>
-
 #include "system.h"
+
+#include <regex.h>
 #include "long-options.h"
 #include "error.h"
+#include "closeout.h"
 
-#define NEW(type) ((type *) xmalloc (sizeof (type)))
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "expr"
+
+#define AUTHORS "Mike Parker"
+
+#undef NEW
+#define NEW(Type) XMALLOC (Type, 1)
 #define OLD(x) free ((char *) x)
 
 /* The kinds of value we can have.  */
@@ -54,7 +61,7 @@ struct valinfo
   TYPE type;			/* Which kind. */
   union
   {				/* The value itself. */
-    int i;
+    intmax_t i;
     char *s;
   } u;
 };
@@ -66,28 +73,12 @@ static char **args;
 /* The name this program was run with. */
 char *program_name;
 
-char *xstrdup ();
-char *strstr ();
-char *xmalloc ();
+static VALUE *eval PARAMS ((void));
+static int nomoreargs PARAMS ((void));
+static int null PARAMS ((VALUE *v));
+static void printv PARAMS ((VALUE *v));
 
-static VALUE *docolon __P ((VALUE *sv, VALUE *pv));
-static VALUE *eval __P ((void));
-static VALUE *int_value __P ((int i));
-static VALUE *str_value __P ((char *s));
-static int isstring __P ((VALUE *v));
-static int nextarg __P ((char *str));
-static int nomoreargs __P ((void));
-static int null __P ((VALUE *v));
-static int toarith __P ((VALUE *v));
-static void freev __P ((VALUE *v));
-static void printv __P ((VALUE *v));
-static void tostring __P ((VALUE *v));
-
-#ifdef EVAL_TRACE
-static void trace ();
-#endif
-
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -100,19 +91,19 @@ Usage: %s EXPRESSION\n\
   or:  %s OPTION\n\
 "),
 	      program_name, program_name);
-      printf (_("\
+      fputs ("\n", stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
-  --help      display this help and exit\n\
-  --version   output version information and exit\n\
-\n\
-"));
-      printf (_("\
 Print the value of EXPRESSION to standard output.  A blank line below\n\
 separates increasing precedence groups.  EXPRESSION may be:\n\
 \n\
   ARG1 | ARG2       ARG1 if it is neither null nor 0, otherwise ARG2\n\
 \n\
   ARG1 & ARG2       ARG1 if neither argument is null or 0, otherwise 0\n\
+"), stdout);
+      fputs (_("\
 \n\
   ARG1 < ARG2       ARG1 is less than ARG2\n\
   ARG1 <= ARG2      ARG1 is less than or equal to ARG2\n\
@@ -120,13 +111,19 @@ separates increasing precedence groups.  EXPRESSION may be:\n\
   ARG1 != ARG2      ARG1 is unequal to ARG2\n\
   ARG1 >= ARG2      ARG1 is greater than or equal to ARG2\n\
   ARG1 > ARG2       ARG1 is greater than ARG2\n\
+"), stdout);
+      fputs (_("\
 \n\
   ARG1 + ARG2       arithmetic sum of ARG1 and ARG2\n\
   ARG1 - ARG2       arithmetic difference of ARG1 and ARG2\n\
+"), stdout);
+      fputs (_("\
 \n\
   ARG1 * ARG2       arithmetic product of ARG1 and ARG2\n\
   ARG1 / ARG2       arithmetic quotient of ARG1 divided by ARG2\n\
-  ARG1 %% ARG2       arithmetic remainder of ARG1 divided by ARG2\n\
+  ARG1 % ARG2       arithmetic remainder of ARG1 divided by ARG2\n\
+"), stdout);
+      fputs (_("\
 \n\
   STRING : REGEXP   anchored pattern match of REGEXP in STRING\n\
 \n\
@@ -134,17 +131,21 @@ separates increasing precedence groups.  EXPRESSION may be:\n\
   substr STRING POS LENGTH   substring of STRING, POS counted from 1\n\
   index STRING CHARS         index in STRING where any CHARS is found, or 0\n\
   length STRING              length of STRING\n\
+"), stdout);
+      fputs (_("\
+  + TOKEN                    interpret TOKEN as a string, even if it is a\n\
+                               keyword like `match' or an operator like `/'\n\
 \n\
   ( EXPRESSION )             value of EXPRESSION\n\
-"));
-      printf (_("\
+"), stdout);
+      fputs (_("\
 \n\
 Beware that many operators need to be escaped or quoted for shells.\n\
 Comparisons are arithmetic if both ARGs are numbers, else lexicographical.\n\
 Pattern matches return the string matched between \\( and \\) or null; if\n\
 \\( and \\) are not used, they return the number of characters matched or 0.\n\
-"));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -159,9 +160,17 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
-  if (getenv ("POSIXLY_CORRECT") == NULL)
-    parse_long_options (argc, argv, "expr", GNU_PACKAGE, VERSION, usage);
+  atexit (close_stdout);
+
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
+  /* The above handles --help and --version.
+     Since there is no other invocation of getopt, handle `--' here.  */
+  if (argc > 1 && STREQ (argv[1], "--"))
+    {
+      --argc;
+      ++argv;
+    }
 
   if (argc == 1)
     {
@@ -182,7 +191,7 @@ main (int argc, char **argv)
 /* Return a VALUE for I.  */
 
 static VALUE *
-int_value (int i)
+int_value (intmax_t i)
 {
   VALUE *v;
 
@@ -215,22 +224,46 @@ freev (VALUE *v)
   OLD (v);
 }
 
+/* Store a printable representation of I somewhere into BUF, and
+   return a pointer to the stored representation.  */
+
+static char *
+inttostr (intmax_t i, char buf[INT_STRLEN_BOUND (intmax_t) + 1])
+{
+  uintmax_t ui = i;
+  char *p = buf + INT_STRLEN_BOUND (intmax_t);
+  *p = '\0';
+  if (i < 0)
+    ui = -ui;
+  do
+    *--p = '0' + ui % 10;
+  while ((ui /= 10) != 0);
+  if (i < 0)
+    *--p = '-';
+  return p;
+}
+
 /* Print VALUE V.  */
 
 static void
 printv (VALUE *v)
 {
+  char *p;
+  char buf[INT_STRLEN_BOUND (intmax_t) + 1];
+
   switch (v->type)
     {
     case integer:
-      printf ("%d\n", v->u.i);
+      p = inttostr (v->u.i, buf);
       break;
     case string:
-      printf ("%s\n", v->u.s);
+      p = v->u.s;
       break;
     default:
       abort ();
     }
+
+  puts (p);
 }
 
 /* Return nonzero if V is a null-string or zero-number.  */
@@ -243,33 +276,23 @@ null (VALUE *v)
     case integer:
       return v->u.i == 0;
     case string:
-      return v->u.s[0] == '\0' || strcmp(v->u.s, "0") == 0;
+      return v->u.s[0] == '\0' || strcmp (v->u.s, "0") == 0;
     default:
       abort ();
     }
 }
 
-/* Return nonzero if V is a string value.  */
-
-static int
-isstring (VALUE *v)
-{
-  return v->type == string;
-}
-
 /* Coerce V to a string value (can't fail).  */
 
 static void
 tostring (VALUE *v)
 {
-  char *temp;
+  char buf[INT_STRLEN_BOUND (intmax_t) + 1];
 
   switch (v->type)
     {
     case integer:
-      temp = xmalloc (4 * (sizeof (int) / sizeof (char)));
-      sprintf (temp, "%d", v->u.i);
-      v->u.s = temp;
+      v->u.s = xstrdup (inttostr (v->u.i, buf));
       v->type = string;
       break;
     case string:
@@ -284,7 +307,7 @@ tostring (VALUE *v)
 static int
 toarith (VALUE *v)
 {
-  int i;
+  intmax_t i;
   int neg;
   char *cp;
 
@@ -295,19 +318,19 @@ toarith (VALUE *v)
     case string:
       i = 0;
       cp = v->u.s;
-      /* Don't interpret the empty string as an integer.  */
-      if (*cp == 0)
-	return 0;
       neg = (*cp == '-');
       if (neg)
 	cp++;
-      for (; *cp; cp++)
+
+      do
 	{
 	  if (ISDIGIT (*cp))
 	    i = i * 10 + *cp - '0';
 	  else
 	    return 0;
 	}
+      while (*++cp);
+
       free (v->u.s);
       v->u.i = i * (neg ? -1 : 1);
       v->type = integer;
@@ -317,7 +340,7 @@ toarith (VALUE *v)
     }
 }
 
-/* Return nonzero if the next token matches STR exactly.
+/* Return nonzero and advance if the next token matches STR exactly.
    STR must not be NULL.  */
 
 static int
@@ -325,7 +348,12 @@ nextarg (char *str)
 {
   if (*args == NULL)
     return 0;
-  return strcmp (*args, str) == 0;
+  else
+    {
+      int r = strcoll (*args, str) == 0;
+      args += r;
+      return r;
+    }
 }
 
 /* Return nonzero if there no more tokens.  */
@@ -336,60 +364,6 @@ nomoreargs (void)
   return *args == 0;
 }
 
-/* The comparison operator handling functions.  */
-
-#define cmpf(name, rel)				\
-static						\
-int name (l, r) VALUE *l; VALUE *r;		\
-{						\
-  if (isstring (l) || isstring (r))		\
-    {						\
-       tostring (l);				\
-       tostring (r);				\
-       return strcmp (l->u.s, r->u.s) rel 0;	\
-    }						\
- else						\
-   return l->u.i rel r->u.i;			\
-}
- cmpf (less_than, <)
- cmpf (less_equal, <=)
- cmpf (equal, ==)
- cmpf (not_equal, !=)
- cmpf (greater_equal, >=)
- cmpf (greater_than, >)
-
-#undef cmpf
-
-/* The arithmetic operator handling functions.  */
-
-#define arithf(name, op)			\
-static						\
-int name (l, r) VALUE *l; VALUE *r;		\
-{						\
-  if (!toarith (l) || !toarith (r))		\
-    error (2, 0, _("non-numeric argument"));	\
-  return l->u.i op r->u.i;			\
-}
-
-#define arithdivf(name, op)			\
-int name (l, r) VALUE *l; VALUE *r;		\
-{						\
-  if (!toarith (l) || !toarith (r))		\
-    error (2, 0, _("non-numeric argument"));	\
-  if (r->u.i == 0)				\
-    error (2, 0, _("division by zero"));		\
-  return l->u.i op r->u.i;			\
-}
-
- arithf (plus, +)
- arithf (minus, -)
- arithf (multiply, *)
- arithdivf (divide, /)
- arithdivf (mod, %)
-
-#undef arithf
-#undef arithdivf
-
 #ifdef EVAL_TRACE
 /* Print evaluation trace and args remaining.  */
 
@@ -417,7 +391,8 @@ docolon (VALUE *sv, VALUE *pv)
   const char *errmsg;
   struct re_pattern_buffer re_buffer;
   struct re_registers re_regs;
-  int len;
+  size_t len;
+  int matchlen;
 
   tostring (sv);
   tostring (pv);
@@ -434,6 +409,8 @@ of the basic regular expression is not portable; it is being ignored"),
   memset (&re_buffer, 0, sizeof (re_buffer));
   memset (&re_regs, 0, sizeof (re_regs));
   re_buffer.allocated = 2 * len;
+  if (re_buffer.allocated < len)
+    xalloc_die ();
   re_buffer.buffer = (unsigned char *) xmalloc (re_buffer.allocated);
   re_buffer.translate = 0;
   re_syntax_options = RE_SYNTAX_POSIX_BASIC;
@@ -441,8 +418,8 @@ of the basic regular expression is not portable; it is being ignored"),
   if (errmsg)
     error (2, 0, "%s", errmsg);
 
-  len = re_match (&re_buffer, sv->u.s, strlen (sv->u.s), 0, &re_regs);
-  if (len >= 0)
+  matchlen = re_match (&re_buffer, sv->u.s, strlen (sv->u.s), 0, &re_regs);
+  if (0 <= matchlen)
     {
       /* Were \(...\) used? */
       if (re_buffer.re_nsub > 0)/* was (re_regs.start[1] >= 0) */
@@ -451,12 +428,12 @@ of the basic regular expression is not portable; it is being ignored"),
 	  v = str_value (sv->u.s + re_regs.start[1]);
 	}
       else
-	v = int_value (len);
+	v = int_value (matchlen);
     }
   else
     {
       /* Match failed -- return the right kind of null.  */
-      if (strstr (pv->u.s, "\\("))
+      if (re_buffer.re_nsub > 0)
 	v = str_value ("");
       else
 	v = int_value (0);
@@ -480,11 +457,9 @@ eval7 (void)
 
   if (nextarg ("("))
     {
-      args++;
       v = eval ();
       if (!nextarg (")"))
 	error (2, 0, _("syntax error"));
-      args++;
       return v;
     }
 
@@ -494,7 +469,7 @@ eval7 (void)
   return str_value (*args++);
 }
 
-/* Handle match, substr, index, and length keywords.  */
+/* Handle match, substr, index, and length keywords, and quoting "+".  */
 
 static VALUE *
 eval6 (void)
@@ -508,9 +483,14 @@ eval6 (void)
 #ifdef EVAL_TRACE
   trace ("eval6");
 #endif
-  if (nextarg ("length"))
+  if (nextarg ("+"))
+    {
+      if (nomoreargs ())
+	error (2, 0, _("syntax error"));
+      return str_value (*args++);
+    }
+  else if (nextarg ("length"))
     {
-      args++;
       r = eval6 ();
       tostring (r);
       v = int_value (strlen (r->u.s));
@@ -519,7 +499,6 @@ eval6 (void)
     }
   else if (nextarg ("match"))
     {
-      args++;
       l = eval6 ();
       r = eval6 ();
       v = docolon (l, r);
@@ -529,13 +508,12 @@ eval6 (void)
     }
   else if (nextarg ("index"))
     {
-      args++;
       l = eval6 ();
       r = eval6 ();
       tostring (l);
       tostring (r);
       v = int_value (strcspn (l->u.s, r->u.s) + 1);
-      if (v->u.i == (int) strlen (l->u.s) + 1)
+      if (v->u.i == strlen (l->u.s) + 1)
 	v->u.i = 0;
       freev (l);
       freev (r);
@@ -543,13 +521,12 @@ eval6 (void)
     }
   else if (nextarg ("substr"))
     {
-      args++;
       l = eval6 ();
       i1 = eval6 ();
       i2 = eval6 ();
       tostring (l);
       if (!toarith (i1) || !toarith (i2)
-	  || i1->u.i > (int) strlen (l->u.s)
+	  || strlen (l->u.s) < i1->u.i
 	  || i1->u.i <= 0 || i2->u.i <= 0)
 	v = str_value ("");
       else
@@ -587,7 +564,6 @@ eval5 (void)
     {
       if (nextarg (":"))
 	{
-	  args++;
 	  r = eval6 ();
 	  v = docolon (l, r);
 	  freev (l);
@@ -606,8 +582,8 @@ eval4 (void)
 {
   VALUE *l;
   VALUE *r;
-  int (*fxn) ();
-  int val;
+  enum { multiply, divide, mod } fxn;
+  intmax_t val;
 
 #ifdef EVAL_TRACE
   trace ("eval4");
@@ -623,9 +599,17 @@ eval4 (void)
 	fxn = mod;
       else
 	return l;
-      args++;
       r = eval5 ();
-      val = (*fxn) (l, r);
+      if (!toarith (l) || !toarith (r))
+	error (2, 0, _("non-numeric argument"));
+      if (fxn == multiply)
+	val = l->u.i * r->u.i;
+      else
+	{
+	  if (r->u.i == 0)
+	    error (2, 0, _("division by zero"));
+	  val = fxn == divide ? l->u.i / r->u.i : l->u.i % r->u.i;
+	}
       freev (l);
       freev (r);
       l = int_value (val);
@@ -639,8 +623,8 @@ eval3 (void)
 {
   VALUE *l;
   VALUE *r;
-  int (*fxn) ();
-  int val;
+  enum { plus, minus } fxn;
+  intmax_t val;
 
 #ifdef EVAL_TRACE
   trace ("eval3");
@@ -654,9 +638,10 @@ eval3 (void)
 	fxn = minus;
       else
 	return l;
-      args++;
       r = eval4 ();
-      val = (*fxn) (l, r);
+      if (!toarith (l) || !toarith (r))
+	error (2, 0, _("non-numeric argument"));
+      val = fxn == plus ? l->u.i + r->u.i : l->u.i - r->u.i;
       freev (l);
       freev (r);
       l = int_value (val);
@@ -670,8 +655,13 @@ eval2 (void)
 {
   VALUE *l;
   VALUE *r;
-  int (*fxn) ();
+  enum
+  {
+    less_than, less_equal, equal, not_equal, greater_equal, greater_than
+  } fxn;
   int val;
+  intmax_t lval;
+  intmax_t rval;
 
 #ifdef EVAL_TRACE
   trace ("eval2");
@@ -693,11 +683,26 @@ eval2 (void)
 	fxn = greater_than;
       else
 	return l;
-      args++;
       r = eval3 ();
-      toarith (l);
-      toarith (r);
-      val = (*fxn) (l, r);
+      tostring (l);
+      tostring (r);
+      lval = strcoll (l->u.s, r->u.s);
+      rval = 0;
+      if (toarith (l) && toarith (r))
+	{
+	  lval = l->u.i;
+	  rval = r->u.i;
+	}
+      switch (fxn)
+	{
+	case less_than:     val = (lval <  rval); break;
+	case less_equal:    val = (lval <= rval); break;
+	case equal:         val = (lval == rval); break;
+	case not_equal:     val = (lval != rval); break;
+	case greater_equal: val = (lval >= rval); break;
+	case greater_than:  val = (lval >  rval); break;
+	default: abort ();
+	}
       freev (l);
       freev (r);
       l = int_value (val);
@@ -720,7 +725,6 @@ eval1 (void)
     {
       if (nextarg ("&"))
 	{
-	  args++;
 	  r = eval2 ();
 	  if (null (l) || null (r))
 	    {
@@ -752,7 +756,6 @@ eval (void)
     {
       if (nextarg ("|"))
 	{
-	  args++;
 	  r = eval1 ();
 	  if (null (l))
 	    {
