@@ -1,5 +1,5 @@
-/* factor -- print factors of n.
-   Copyright (C) 86, 95, 96, 1997, 1998, 1999 Free Software Foundation, Inc.
+/* factor -- print prime factors of n.
+   Copyright (C) 86, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,9 +26,10 @@
 #define NDEBUG 1
 
 #include "system.h"
-#include "long-options.h"
+#include "closeout.h"
 #include "error.h"
 #include "human.h"
+#include "long-options.h"
 #include "readtokens.h"
 #include "xstrtol.h"
 
@@ -44,6 +45,21 @@
    than 2^128, this constant (and the algorithm :-) will have to change.  */
 #define MAX_N_FACTORS 128
 
+/* The trial divisor increment wheel.  Use it to skip over divisors that
+   are composites of 2, 3, 5, 7, or 11.  The part from WHEEL_START up to
+   WHEEL_END is reused periodically, while the "lead in" is used to test
+   for those primes and to jump onto the wheel.  For more information, see
+   http://www.utm.edu/research/primes/glossary/WheelFactorization.html  */
+
+#include "wheel-size.h"  /* For the definition of WHEEL_SIZE.  */
+static const unsigned int wheel_tab[] =
+  {
+#include "wheel.h"
+  };
+
+#define WHEEL_START (wheel_tab + WHEEL_SIZE)
+#define WHEEL_END (wheel_tab + (sizeof wheel_tab / sizeof wheel_tab[0]))
+
 /* The name this program was run with. */
 char *program_name;
 
@@ -60,16 +76,18 @@ Usage: %s [NUMBER]...\n\
   or:  %s OPTION\n\
 "),
 	      program_name, program_name);
-      printf (_("\
-Print factors of each NUMBER; read standard input with no arguments.\n\
+      fputs (_("\
+Print the prime factors of each NUMBER.\n\
 \n\
-  --help      display this help and exit\n\
-  --version   output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
   Print the prime factors of all specified integer NUMBERs.  If no arguments\n\
   are specified on the command line, they are read from standard input.\n\
-"));
-      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -81,17 +99,11 @@ factor (uintmax_t n0, int max_n_factors, uintmax_t *factors)
 {
   register uintmax_t n = n0, d, q;
   int n_factors = 0;
+  unsigned int const *w = wheel_tab;
 
   if (n < 1)
     return n_factors;
 
-  while (n % 2 == 0)
-    {
-      assert (n_factors < max_n_factors);
-      factors[n_factors++] = 2;
-      n /= 2;
-    }
-
   /* The exit condition in the following loop is correct because
      any time it is tested one of these 3 conditions holds:
       (1) d divides n
@@ -100,7 +112,7 @@ factor (uintmax_t n0, int max_n_factors, uintmax_t *factors)
      If (1) or (2) obviously the right thing happens.
      If (3), then since n is composite it is >= d^2. */
 
-  d = 3;
+  d = 2;
   do
     {
       q = n / d;
@@ -111,7 +123,9 @@ factor (uintmax_t n0, int max_n_factors, uintmax_t *factors)
 	  n = q;
 	  q = n / d;
 	}
-      d += 2;
+      d += *(w++);
+      if (w == WHEEL_END)
+	w = WHEEL_START;
     }
   while (d <= q);
 
@@ -181,6 +195,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
 		      AUTHORS, usage);
   /* The above handles --help and --version.
