@@ -1,5 +1,5 @@
 /* logname -- print user's login name
-   Copyright (C) 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1990-1997, 1999, 2000, 2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,24 +21,23 @@
 #include <getopt.h>
 
 #include "system.h"
+#include "closeout.h"
+#include "long-options.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "logname"
+
+#define AUTHORS "FIXME: unknown"
 
 /* The name this program was run with. */
 char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output and exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -46,14 +45,14 @@ usage (int status)
 	     program_name);
   else
     {
-      printf (_("Usage: %s [OPTION]...\n"), program_name);
-      printf (_("\
+      printf (_("Usage: %s [OPTION]\n"), program_name);
+      fputs (_("\
 Print the name of the current user.\n\
 \n\
-  --help      display this help and exit\n\
-  --version   output version information and exit\n\
-"));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -69,7 +68,12 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while ((c = getopt_long (argc, argv, "", long_options, (int *) 0)) != EOF)
+  atexit (close_stdout);
+
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
+
+  while ((c = getopt_long (argc, argv, "", long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -81,26 +85,17 @@ main (int argc, char **argv)
 	}
     }
 
-  if (show_version)
-    {
-      printf ("logname (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (argc - optind != 0)
     usage (1);
 
-  /* POSIX.2 requires using getlogin (or equivalent code).  */
+  /* POSIX requires using getlogin (or equivalent code).  */
   cp = getlogin ();
   if (cp)
     {
       puts (cp);
       exit (0);
     }
-  /* POSIX.2 prohibits using a fallback technique.  */
-  fprintf (stderr,_("%s: no login name\n"), argv[0]);
+  /* POSIX prohibits using a fallback technique.  */
+  fprintf (stderr, _("%s: no login name\n"), argv[0]);
   exit (1);
 }
