@@ -1,5 +1,5 @@
 /* GNU's users.
-   Copyright (C) 1992-1999 Free Software Foundation, Inc.
+   Copyright (C) 1992-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,15 +21,18 @@
 #include <getopt.h>
 #include <stdio.h>
 
+#include <sys/types.h>
+#include "system.h"
+
 #include "error.h"
 #include "long-options.h"
 #include "readutmp.h"
-#include "system.h"
+#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "users"
 
-#define AUTHORS "Joseph Arceneaux and David MacKenzie"
+#define AUTHORS N_ ("Joseph Arceneaux and David MacKenzie")
 
 /* The name this program was run with. */
 char *program_name;
@@ -58,7 +61,7 @@ list_entries_users (int n, const STRUCT_UTMP *this)
   u = (char **) xmalloc (n * sizeof (u[0]));
   for (i = 0; i < n; i++)
     {
-      if (this->ut_name[0]
+      if (UT_USER (this) [0]
 #ifdef USER_PROCESS
 	  && this->ut_type == USER_PROCESS
 #endif
@@ -117,10 +120,11 @@ usage (int status)
 Output who is currently logged in according to FILE.\n\
 If FILE is not specified, use %s.  %s as FILE is common.\n\
 \n\
-      --help        display this help and exit\n\
-      --version     output version information and exit\n"),
+"),
 	      UTMP_FILE, WTMP_FILE);
-      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -134,6 +138,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
 		      AUTHORS, usage);
 
